/**
 * Date:            Time:       User:               Description:
 * 14.03.2006       20:19       Smart Max           creation of the file IPatriciaTrie.java
 *
 */
package org.max.trie;

import java.util.List;

/**
 * IPatriciaTrie
 * <p>
 * For all setters go to {@link IWritablePatriciaTrie}
 *
 * @author  Maxim Podkolzine
 */
public interface IPatriciaTrie {
    /**
     * Returns the root of the trie
     *
     * @return                                      the root
     */
    public INode getRoot();

    /**
     * Returns the node of the trie corresponding the specified string.
     * <p>
     * Searching in trie lies in comparing characters in the search key (<tt>string</tt>) to edge labels,
     * and following those edges. Note, that not all symbols from key are significant (the others can
     * arbitrary change, and the result will not vary).
     * <p>
     * Consider, for example, a trie that consists of three input strings: "cash", "cat", "fast". The root node
     * has two children by symbols 'f' and 'c' with depths <tt>4</tt> and <tt>2</tt>, and the last
     * has two leaf nodes by symbols 's' and 't' with depths <tt>4</tt> and <tt>3</tt>.
     * Searching in this trie will pass by the following steps:
     * <ul>
     * <li> if the key is empty (the length equals to zero), the root node is returned
     * <li> otherwise the first symbol of <tt>string</tt> will be compared: if it is not 'c' and not 'f',
     *      the key is not found
     * <li> if the first symbol is 'f', then the length of key is compared: in case of <tt>4</tt> the
     *      leaf node "fast" is returned, otherwise, the key is not found
     * <li> if the first symbol is 'c', then the searching proceeds from node with depth <tt>2</tt> and
     *      several cases are possible:
     * <li> if the key length is <tt>2</tt>, method returns internal node with depth <tt>2</tt>
     * <li> otherwise the third symbol is compared: if it is not 's' and not 't', the key is not found
     * <li> else depending on the length of the key <tt>string</tt> the result can be "cash", "cat" or
     *      <tt>null</tt>.
     * </ul>
     * Here, the significant is the first symbol of input key and possibly the third one. The others can
     * vary and the result will remain if only the length of key is proper.
     * <p>
     * By contract, if the key is not found, method returns <tt>null</tt>
     *
     * @param   string                              the input search key
     * @return                                      the node of the trie corresponding the specified string
     */
    public INode get(List<ISymbol> string);

    /**
     * Returns the size of the trie (i.e. the number of nodes in it)
     *
     * @return                                      the trie size
     */
    public int size();
}
