/**
 * Date:            Time:       User:               Description:
 * 04.05.2006       21:59       Smart Max           creation of the file IndexFabricSerializer.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;

import org.max.trie.impl.PatriciaTrieSerializer;

import java.io.*;

import net.sf.xanswer.indexing.IndexException;

/**
 * IndexFabricSerializer
 *
 * @author  Maxim Podkolzine
 */
public class IndexFabricSerializer {
    // Data

    /**
     * Holds the Index Fabric to serialize
     */
    private IndexFabric myIndexFabric;

    /**
     * Holds the destination location
     */
    private String myLocation;




    // Constructor

    /**
     * Creates the serializer with specified IndexFabric and location
     *
     * @param   indexFabric                         the Index Fabric needed be to serialized
     * @param   location                            the location where to serialize
     */
    public IndexFabricSerializer(IndexFabric indexFabric, String location) {
        myIndexFabric = indexFabric;
        myLocation = location;
    }




    // Main interface

    /**
     * Writes the Index Fabric to the specified location.
     * <p>
     * Both Index Fabric and location are passed in constructor. Method serializes the maps of designators
     * to the certain file and then serializes the Patricia trie. Note that the trie is written without
     * the map of symbols on edges (see {@link PatriciaTrieSerializer#writeTrie()}). Thus after deserialization
     * it does not support modifications.
     *
     * @throws  IndexException                      if errors while serialization occur
     */
    public void writeFabric() throws IndexException {
        try {
            ObjectOutput output = new ObjectOutputStream(newOutputStream(myLocation + Separator + "symbols.dat"));
            output.writeObject(myIndexFabric.getAttrMap());
            output.writeObject(myIndexFabric.getTextMap());
            output.writeObject(myIndexFabric.getTagMap());
            output.close();

            PatriciaTrieSerializer trieSerializer = new PatriciaTrieSerializer(myIndexFabric.getTrie(), myLocation);
            trieSerializer.writeTrie();
        } catch (IOException e) {
            throw new IndexException(e);
        }
    }
}
