/**
 * Date:            Time:       User:               Description:
 * 01.04.2006       17:13       Smart Max           creation of the file IndexFabricTest.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;
import static org.max.xml.index.TestUtil.*;
import static java.util.Arrays.*;

import junit.framework.TestCase;

import java.util.ArrayList;


import org.max.trie.IWritablePatriciaTrie;
import org.max.trie.ISymbol;
import net.sf.xanswer.indexing.IndexException;


/**
 * IndexFabricTest
 */
public class IndexFabricTest extends TestCase {
    public void test_Simple() throws IndexException {
        String docID1 = "xml" + Separator + "doc1.xml";
        String docID2 = "xml" + Separator + "doc2.xml";

        XMLBuilder builder = new XMLBuilder();
        XMLBuilder.restoreDefaults();
        builder.addXMLDocument(docID1);
        builder.addXMLDocument(docID2);
        IndexFabric fabric = builder.getIndexFabric();

        try {
            IWritablePatriciaTrie trie = (IWritablePatriciaTrie) fabric.getTrie();
            trie.add(new ArrayList<ISymbol>());
            fail("No addings must be possible");
        } catch (UnsupportedOperationException e) {
        }

        XMLRetriever retriever = new XMLRetriever(fabric);

        checkLeafNode(retriever.retrieveRawPath(asList("invoice", "buyer", "name"), "ABC Corp"), docID1);
        checkLeafNode(retriever.retrieveRawPath(asList("invoice", "buyer", "address"), "1 Industrial Way"), docID1);
        checkLeafNode(retriever.retrieveRawPath(asList("invoice", "buyer", "name"), "Oracle Inc"), docID2);
        checkLeafNode(retriever.retrieveRawPath(asList("invoice", "buyer", "phone"), "555-1212"), docID2);
    }

    public void test_SaveAndUpload1() throws IndexException {
        String[] docIDs = new String[]{
                "xml" + Separator + "WinDes.xml",
                "xml" + Separator + "acrobat.xml",
                "xml" + Separator + "XRXS1um.xml"
        };

        XMLBuilder builder = new XMLBuilder();
        for (String docID : docIDs) {
            builder.addXMLDocument(docID);
        }

        String location = "test" + Separator + "SaveAndUpload1";
        createDirectory(location);

        IndexFabricSerializer serializer = new IndexFabricSerializer(builder.getIndexFabric(), location);
        serializer.writeFabric();

        IndexFabricDeserializer deserializer = new IndexFabricDeserializer();
        IndexFabric fabric = deserializer.readIndexFabric(location);

        XMLRetriever retriever = new XMLRetriever(fabric);


        checkLeafNode(retriever.retrieveRawPath(asList("doc", "assembly", "name"), "WinDes"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "T:Microsoft.Tools.WinDes.CustomItemDialog"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "summary", "para", "see"), "cref",
                        "T:Microsoft.Tools.WinDes.DocumentCollection"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "param"), "name", "value"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "summary", "para"), "Adds a "), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "returns", "para"), "None."), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "param"), "name", "index"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "param", "paramref"), "name", "value"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member", "param"), " should be inserted."), docIDs[0]);
        checkInternalNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "M:Microsoft.Tools.WinDes.FileBuffer.ShowCode"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "M:Microsoft.Tools.WinDes.FileBuffer.ShowCode(System.Int32)"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name", "M:Microsoft.Tools.WinDes.InheritedForm.#ctor"),
                docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "M:Microsoft.Tools.WinDes.SelfHost.Dispose(System.Boolean)"),docIDs[0]);
        checkInternalNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "T:Microsoft.Tools.WinDes.TestCommand"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "T:Microsoft.Tools.WinDes.TestCommandScript"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "M:Microsoft.Tools.WinDes.TestCommandScript.Run"), docIDs[0]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("doc", "members", "member"), "name",
                        "T:Microsoft.Tools.WinDes.WinDesExtension"), docIDs[0]);


        checkLeafNode(retriever.retrieveRawPath(asList("xsl:stylesheet"), "version", "1.0"), docIDs[1]);
        checkLeafNode(retriever.retrieveRawPath(asList("xsl:stylesheet", "xsl:output"), "method", "xml"),
                docIDs[1]);
        checkLeafNode(retriever.retrieveRawPath(asList("xsl:stylesheet", "xsl:output"), "encoding", "UTF-8"),
                docIDs[1]);
        checkLeafNode(retriever.retrieveRawPath(asList("xsl:stylesheet", "xsl:output"), "indent", "no"),
                docIDs[1]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("xsl:stylesheet", "xsl:output"), "omit-xml-declaration", "no"), docIDs[1]);
        //...


        checkLeafNode(retriever.retrieveRawPath(asList("Device"), "name", "Unified FB"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(asList("Device", "Capability", "Emulation"), "type", "2"),
                docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(asList("Device", "Capability", "PJL", "MEDIATYPE")));
        checkLeafNode(retriever.retrieveRawPath(asList("Device", "Capability", "PJL", "ALTITUDE")));
        checkLeafNode(retriever.retrieveRawPath(asList("Device", "Capability", "PJL", "TONERSAVE")));
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "ResMapping", "mapping"), "iResolution", "600"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "ResMapping", "mapping", "set"), "PrintBitsPerPixel", "1"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Colors", "LinearTable", "priority"), "PrintResX", "0"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Colors", "LinearTable", "group", "cond", "PrintResX", "opt"), "val", "600"),
                docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "Keys", "MEDIATYPE"), "id", "0"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "Keys", "dmPrintQuality"), "id", "3"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "Keys", "PrintBitsPerPixel"), "id", "16"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "DUPLEX", "opt"), "name", "AUTODUPLEX_LONGEDGE"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "DUPLEX", "opt", "cmd"),
                        "@PJL SET DUPLEX=ON"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "JAMRECOVERY", "opt", "cmd"),
                        "@PJL SET JAMRECOVERY = ON"), docIDs[2]);
        checkLeafNode(retriever.retrieveRawPath(
                asList("Device", "Definitions", "JOBRETENTION", "opt"), "name", "JOBRETENTION_PROOF&STORE"),
                docIDs[2]);
    }
}
